{-# LANGUAGE CApiFFI #-}
{-# LANGUAGE ForeignFunctionInterface #-}
{-# LANGUAGE GeneralizedNewtypeDeriving #-}
{-# OPTIONS_GHC -optc -Wno-discarded-qualifiers #-}

module OkCrypt.Cdefs where

import Foreign.C.String
import Foreign.C.Types
import OkStd

-- | Wrapper for error codes
newtype Cerr = Cerr CInt deriving (Eq, Show, Enum, Ord, Num, Real, Integral)

sodiumSuccess :: Cerr
sodiumSuccess = Cerr 0

-- | > int sodium_init(void);
foreign import capi "sodium.h sodium_init" sodium_init :: IO Cerr

-- | > void randombytes_buf(void * const buf, const size_t size);
foreign import capi "sodium.h randombytes_buf" randombytes_buf :: CString -> CSize -> IO ()

-- ** Authenticated Encryption (Secret Key)

foreign import capi "value crypto_secretbox_KEYBYTES" crypto_secretbox_KEYBYTES :: CSize

foreign import capi "value crypto_secretbox_NONCEBYTES" crypto_secretbox_NONCEBYTES :: CSize

foreign import capi "value crypto_secretbox_MACBYTES" crypto_secretbox_MACBYTES :: CSize

-- | > void crypto_secretbox_keygen(unsigned char k[crypto_secretbox_KEYBYTES]);
foreign import capi "sodium.h crypto_secretbox_keygen" crypto_secretbox_keygen :: CString -> IO ()

-- |
-- > int crypto_secretbox_easy(unsigned char *c, const unsigned char *m,
-- >                           unsigned long long mlen, const unsigned char *n,
-- >                           const unsigned char *k);
foreign import capi "sodium.h crypto_secretbox_easy"
  crypto_secretbox_easy ::
    CString -> CString -> CLLong -> CString -> CString -> IO Cerr

-- |
-- > int crypto_secretbox_open_easy(unsigned char *m, const unsigned char *c,
-- >                                unsigned long long clen, const unsigned char *n,
-- >                                const unsigned char *k);
foreign import capi "sodium.h crypto_secretbox_open_easy"
  crypto_secretbox_open_easy ::
    CString -> CString -> CLLong -> CString -> CString -> IO Cerr

-- ** Key Derivation

foreign import capi "value crypto_pwhash_SALTBYTES" crypto_pwhash_SALTBYTES :: CSize

foreign import capi "value crypto_pwhash_PASSWD_MIN" crypto_pwhash_PASSWD_MIN :: CLLong

foreign import capi "value crypto_pwhash_PASSWD_MAX" crypto_pwhash_PASSWD_MAX :: CLLong

foreign import capi "value crypto_pwhash_OPSLIMIT_INTERACTIVE" crypto_pwhash_OPSLIMIT_INTERACTIVE :: CLLong

foreign import capi "value crypto_pwhash_OPSLIMIT_MODERATE" crypto_pwhash_OPSLIMIT_MODERATE :: CLLong

foreign import capi "value crypto_pwhash_OPSLIMIT_SENSITIVE" crypto_pwhash_OPSLIMIT_SENSITIVE :: CLLong

foreign import capi "value crypto_pwhash_MEMLIMIT_INTERACTIVE" crypto_pwhash_MEMLIMIT_INTERACTIVE :: CSize

foreign import capi "value crypto_pwhash_MEMLIMIT_MODERATE" crypto_pwhash_MEMLIMIT_MODERATE :: CSize

foreign import capi "value crypto_pwhash_MEMLIMIT_SENSITIVE" crypto_pwhash_MEMLIMIT_SENSITIVE :: CSize

foreign import capi "value crypto_pwhash_ALG_DEFAULT" crypto_pwhash_ALG_DEFAULT :: CInt

foreign import capi "value crypto_pwhash_ALG_ARGON2I13" crypto_pwhash_ALG_ARGON2I13 :: CInt

foreign import capi "value crypto_pwhash_ALG_ARGON2ID13" crypto_pwhash_ALG_ARGON2ID13 :: CInt

-- |
-- > int
-- > crypto_pwhash(unsigned char * const out, unsigned long long outlen,
-- >               const char * const passwd, unsigned long long passwdlen,
-- >               const unsigned char * const salt,
-- >               unsigned long long opslimit, size_t memlimit, int alg)
foreign import capi "sodium.h crypto_pwhash"
  crypto_pwhash ::
    CString -> CLLong -> CString -> CLLong -> CString -> CLLong -> CSize -> CInt -> IO Cerr

-- ** Hashing

foreign import capi "value crypto_generichash_BYTES" crypto_generichash_BYTES :: CSize

-- |
-- > int crypto_generichash(unsigned char *out, size_t outlen,
-- >                        const unsigned char *in, unsigned long long inlen,
-- >                        const unsigned char *key, size_t keylen);
foreign import capi "sodium.h crypto_generichash"
  crypto_generichash ::
    CString -> CSize -> CString -> CLLong -> CString -> CSize -> IO Cerr
