module TestBindings where

import qualified Data.ByteString as BS
import OkCrypt.Bindings
import OkStd
import qualified OkStd.Encoding.Base64Url as Base64
import Test.HUnit
import qualified Test.QuickCheck as Q
import qualified Test.QuickCheck.Monadic as Q

tests :: [Test]
tests =
  [ "should encrypt and decrypt using secret key" ~: do
      init
      sk <- generateSecretKey
      result <- Q.quickCheckResult $
        Q.monadicIO $ do
          msg <- BS.pack <$> Q.pick Q.arbitrary
          nonce <- Q.run generateNonce
          ciphertext <- Q.run $ encryptSecret (toPlaintext msg) sk nonce
          plaintext <- Q.run $ decryptSecret ciphertext sk nonce
          Q.assert $ msg == toByteString plaintext
      assertBool (Q.output result) (Q.isSuccess result),
    "should derive the same predefined key from given salt and password" ~: do
      let pwd = "testPASS123"
          Right salt = unsafeMaterialFromByteString <$> Base64.decode "-lNZPEzDMyTLsYI00UMBcg"
          Right expectedKey = Base64.decode "rGBH-5zIROASYfm4EuoKqyANy9Q2BA7wzAVdy1PVzMI"
      actualKey <- toByteString <$> deriveSecretKey pwd salt
      assertEqual "derived secret keys" expectedKey actualKey

      {- Slow test ahead. Uncomment when needed.

      , "should derive the same key from the same salt and password" ~: do
          init
          result <- Q.quickCheckResult $ Q.monadicIO $ do
              pwd <- BS.pack <$> Q.pick Q.arbitrary
              salt <- Q.run generateSalt
              sk1 <- Q.run $ deriveSecretKey pwd salt
              sk2 <- Q.run $ deriveSecretKey pwd salt
              Q.assert $ toByteString sk1 == toByteString sk2
          assertBool (Q.output result) (Q.isSuccess result)
      -}
  ]
