{-# LANGUAGE CApiFFI #-}
{-# LANGUAGE EmptyDataDecls #-}
{-# LANGUAGE ForeignFunctionInterface #-}
{-# LANGUAGE GeneralizedNewtypeDeriving #-}
{-# OPTIONS_GHC -optc -Wno-discarded-qualifiers -Wno-dodgy-foreign-imports #-}

module OkSqlite.Cdefs where

import Foreign.C.String
import Foreign.C.Types
import Foreign.Ptr
import OkStd

-- | The database connection object. Created by `sqlite3_open_v2` and destroyed by `sqlite3_close_v2`.
data {-# CTYPE "sqlite3.h" "sqlite3" #-} Csqlite3

-- | The prepared statement object. Created by `sqlite3_prepare_v3` and destroyed by `sqlite3_finalize`.
data {-# CTYPE "sqlite3.h" "sqlite3_stmt" #-} Csqlite3_stmt

-- | Wrapper for error codes
newtype Cerr = Cerr CInt deriving (Eq, Show, Enum, Ord, Num, Real, Integral)

foreign import capi "value SQLITE_OK" c_SQLITE_OK :: Cerr

foreign import capi "value SQLITE_DONE" c_SQLITE_DONE :: Cerr

foreign import capi "value SQLITE_ROW" c_SQLITE_ROW :: Cerr

-- | Wrapper for sqlite3_open_v2 flags
newtype Csqlite3_flags = Csqlite3_flags CInt deriving (Eq, Show, Bits)

foreign import capi "value SQLITE_OPEN_READONLY" c_SQLITE_OPEN_READONLY :: Csqlite3_flags

foreign import capi "value SQLITE_OPEN_READWRITE" c_SQLITE_OPEN_READWRITE :: Csqlite3_flags

foreign import capi "value SQLITE_OPEN_CREATE" c_SQLITE_OPEN_CREATE :: Csqlite3_flags

-- | Wrapper for SQLite types
newtype Csqlite3_type = Csqlite3_type CInt deriving (Eq, Show)

foreign import capi "value SQLITE_INTEGER" c_SQLITE_INTEGER :: Csqlite3_type

foreign import capi "value SQLITE_FLOAT" c_SQLITE_FLOAT :: Csqlite3_type

foreign import capi "value SQLITE_TEXT" c_SQLITE_TEXT :: Csqlite3_type

foreign import capi "value SQLITE_BLOB" c_SQLITE_BLOB :: Csqlite3_type

foreign import capi "value SQLITE_NULL" c_SQLITE_NULL :: Csqlite3_type

foreign import capi "value SQLITE_TRANSIENT" c_SQLITE_TRANSIENT :: FunPtr a

-- |
-- See https://sqlite.org/c3ref/open.html
--
-- > int sqlite3_open_v2(
-- >   const char *filename,   /* Database filename (UTF-8) */
-- >   sqlite3 **ppDb,         /* OUT: SQLite db handle */
-- >   int flags,              /* Flags */
-- >   const char *zVfs        /* Name of VFS module to use */
-- > );
foreign import capi unsafe "sqlite3_open_v2" sqlite3_open_v2 :: CString -> Ptr (Ptr Csqlite3) -> Csqlite3_flags -> CString -> IO Cerr

-- |
-- See https://sqlite.org/c3ref/close.html
--
-- > int sqlite3_close(sqlite3*);
foreign import capi unsafe "sqlite3_close_v2" sqlite3_close_v2 :: Ptr Csqlite3 -> IO Cerr

-- |
-- See https://sqlite.org/c3ref/prepare.html
--
-- > int sqlite3_prepare_v2(
-- >   sqlite3 *db,            /* Database handle */
-- >   const char *zSql,       /* SQL statement, UTF-8 encoded */
-- >   int nByte,              /* Maximum length of zSql in bytes. */
-- >   sqlite3_stmt **ppStmt,  /* OUT: Statement handle */
-- >   const char **pzTail     /* OUT: Pointer to unused portion of zSql */
-- > );
foreign import capi unsafe "sqlite3_prepare_v2" sqlite3_prepare_v2 :: Ptr Csqlite3 -> CString -> CInt -> Ptr (Ptr Csqlite3_stmt) -> CString -> IO Cerr

-- |
-- See https://sqlite.org/c3ref/db_handle.html
--
-- > sqlite3 *sqlite3_db_handle(sqlite3_stmt*);
foreign import capi unsafe "sqlite3_db_handle" sqlite3_db_handle :: Ptr Csqlite3_stmt -> IO (Ptr Csqlite3)

-- |
-- See https://sqlite.org/c3ref/finalize.html
--
-- > int sqlite3_finalize(sqlite3_stmt *pStmt);
foreign import capi unsafe "sqlite3_finalize" sqlite3_finalize :: Ptr Csqlite3_stmt -> IO Cerr

-- |
-- See https://sqlite.org/c3ref/step.html
--
-- > int sqlite3_step(sqlite3_stmt*);
foreign import capi unsafe "sqlite3_step" sqlite3_step :: Ptr Csqlite3_stmt -> IO Cerr

-- |
-- See https://sqlite.org/c3ref/bind_blob.html
--
-- > int sqlite3_bind_blob(sqlite3_stmt *pStmt, int paramIdx, const void* value, int n, void(*)(void*) finalizer);
foreign import capi unsafe "sqlite3_bind_blob" sqlite3_bind_blob :: Ptr Csqlite3_stmt -> CInt -> Ptr a -> CInt -> FunPtr a -> IO Cerr

-- |
-- See https://sqlite.org/c3ref/bind_blob.html
--
-- > int sqlite3_bind_double(sqlite3_stmt *pStmt, int paramIdx, double value);
foreign import capi unsafe "sqlite3_bind_double" sqlite3_bind_double :: Ptr Csqlite3_stmt -> CInt -> CDouble -> IO Cerr

-- |
-- See https://sqlite.org/c3ref/bind_blob.html
--
-- > int sqlite3_bind_int64(sqlite3_stmt *pStmt, int paramIdx, int64 value);
foreign import capi unsafe "sqlite3_bind_int64" sqlite3_bind_int64 :: Ptr Csqlite3_stmt -> CInt -> CIntMax -> IO Cerr

-- |
-- See https://sqlite.org/c3ref/bind_blob.html
--
-- > int sqlite3_bind_null(sqlite3_stmt *pStmt, int paramIdx);
foreign import capi unsafe "sqlite3_bind_null" sqlite3_bind_null :: Ptr Csqlite3_stmt -> CInt -> IO Cerr

-- |
-- See https://sqlite.org/c3ref/bind_blob.html
--
-- > int sqlite3_bind_text(sqlite3_stmt *pStmt, int paramIdx, const char *value, int n, void(*)(void*) finalizer);
foreign import capi unsafe "sqlite3_bind_text" sqlite3_bind_text :: Ptr Csqlite3_stmt -> CInt -> CString -> CInt -> FunPtr a -> IO Cerr

-- |
-- See https://sqlite.org/c3ref/bind_parameter_index.html
--
-- > int sqlite3_bind_parameter_index(sqlite3_stmt*, const char *zName);
foreign import capi unsafe "sqlite3_bind_parameter_index" sqlite3_bind_parameter_index :: Ptr Csqlite3_stmt -> CString -> IO CInt

-- |
-- See https://sqlite.org/c3ref/column_count.html
--
-- > int sqlite3_column_count(sqlite3_stmt*);
foreign import capi unsafe "sqlite3_column_count" sqlite3_column_count :: Ptr Csqlite3_stmt -> IO CInt

-- |
-- See https://sqlite.org/c3ref/column_blob.html
--
-- > const void *sqlite3_column_blob(sqlite3_stmt*, int iCol);
foreign import capi unsafe "sqlite3_column_blob" sqlite3_column_blob :: Ptr Csqlite3_stmt -> CInt -> IO (Ptr a)

-- |
-- See https://sqlite.org/c3ref/column_blob.html
--
-- > double sqlite3_column_double(sqlite3_stmt*, int iCol);
foreign import capi unsafe "sqlite3_column_double" sqlite3_column_double :: Ptr Csqlite3_stmt -> CInt -> IO CDouble

-- |
-- See https://sqlite.org/c3ref/column_blob.html
--
-- > int64 sqlite3_column_int64(sqlite3_stmt*, int iCol);
foreign import capi unsafe "sqlite3_column_int64" sqlite3_column_int64 :: Ptr Csqlite3_stmt -> CInt -> IO CIntMax

-- |
-- See https://sqlite.org/c3ref/column_blob.html
--
-- > const char *sqlite3_column_text(sqlite3_stmt*, int iCol);
foreign import capi unsafe "sqlite3_column_text" sqlite3_column_text :: Ptr Csqlite3_stmt -> CInt -> IO CString

-- |
-- See https://sqlite.org/c3ref/column_blob.html
--
-- > int sqlite3_column_bytes(sqlite3_stmt*, int iCol);
foreign import capi unsafe "sqlite3_column_bytes" sqlite3_column_bytes :: Ptr Csqlite3_stmt -> CInt -> IO CInt

-- |
-- See https://sqlite.org/c3ref/column_blob.html
--
-- > int sqlite3_column_type(sqlite3_stmt*, int iCol);
foreign import capi unsafe "sqlite3_column_type" sqlite3_column_type :: Ptr Csqlite3_stmt -> CInt -> IO Csqlite3_type

-- |
-- See https://sqlite.org/capi3ref.html#sqlite3_errcode
--
-- > const char *sqlite3_errmsg(sqlite3*);
foreign import capi unsafe "sqlite3_errmsg" sqlite3_errmsg :: Ptr Csqlite3 -> IO CString
