module OkSqlite.Exception
  ( ExceptionOkSqlite (..),
  )
where

import OkStd

-- | Exceptions that could be raised in the library functions
data ExceptionOkSqlite
  = -- | @ExceptionSQLite errCode errMsg@ represents SQLite library error
    ExceptionSQLite Int Text
  | -- | Indicates that @sqlite3_open_v2@ function returned NULL (which should never happen)
    ExceptionDatabaseHandleIsNull
  | -- | Signals that the statement is finalized and could not be used anymore
    ExceptionStatementIsFinalized
  | -- | @ExceptionParameterNotFound paramName@ indicates that given named parameter is not present in the query
    ExceptionParameterNotFound Text
  | -- | Signals that query returned no rows, but a single row was expected
    ExceptionNoRowsReturned
  | -- | @ExceptionMigrationCodeMismatch migrationCode codeInDB@ indicates that SQL code of the migration does not match the one applied to the database
    ExceptionMigrationCodeMismatch Text Text
  deriving (Eq, Show)

instance Exception ExceptionOkSqlite
