module TestBase64Url where

import qualified Data.ByteString as BS
import Data.Word (Word8)
import qualified OkStd.Encoding.Base64Url as Base64
import qualified System.Random.SplitMix as SM
import Test.HUnit
import qualified Test.QuickCheck as Q
import Test.QuickCheck.Gen
import Test.QuickCheck.Random (QCGen (..))

instance Q.Arbitrary BS.ByteString where
  arbitrary = MkGen $ \(QCGen g0) size ->
    if size <= 0
      then BS.empty
      else
        let (i, g1) = SM.nextInt g0
            size' = i `mod` size
         in fst (BS.unfoldrN size' gen g1)
    where
      gen :: SM.SMGen -> Maybe (Word8, SM.SMGen)
      gen g = Just (fromIntegral w64, g')
        where
          (w64, g') = SM.nextWord64 g

tests :: [Test]
tests =
  [ "should encode and decode Base64 URL-safe" ~: do
      Q.quickCheck (\inp -> Right inp == (Base64.decode $ Base64.encode inp))
  ]
