module OkVault.Actions.FirstTime
  ( showFirstTimePage,
    performFirstTimeSetup,
  )
where

import qualified Network.HTTP.Types as HTTP
import OkStd
import OkVault.Actions.Error
import OkVault.Actions.Shared
import OkVault.Context
import qualified OkVault.Models.Session as Session
import qualified OkVault.Models.User as User
import qualified OkVault.Templates as Templates

showFirstTimePage :: Action
showFirstTimePage =
  whenAuthenticated $ \s u ->
    if User.hasKeys u
      then redirectTemporarily "/"
      else showHTML HTTP.status200 $ Templates.firstTimePage (Session.csrfToken s)

performFirstTimeSetup :: Action
performFirstTimeSetup =
  whenCSRFTokenValid $ \params -> do
    case sequence [lookup "salt" params, lookup "data_key" params, lookup "metadata_key" params] of
      Just [salt, dataKey, metadataKey] -> do
        Just user <- currentUser
        withDB (User.setKeys (User.id user) salt dataKey metadataKey)
        redirectTemporarily "/"
      _ -> do
        logDebug "performFirstTimeSetup: missing parameters"
        showMalformedRequestPage
