module OkVault.Actions.Static
  ( staticFile,
  )
where

import qualified Data.List as List
import qualified Data.Text as T
import qualified Network.HTTP.Types as HTTP
import Network.Mime (defaultMimeLookup)
import qualified Network.Wai as Wai
import OkStd
import OkVault.Actions.Error
import OkVault.Actions.Shared
import OkVault.Context
import OkVault.Resources
import System.IO.Error (isDoesNotExistError)

staticFile :: Action
staticFile = do
  -- Match "static" prefix explicitly to prevent access to other resources
  let ("static" : pathParts) = Wai.pathInfo ?req
      path = T.intercalate "/" pathParts
      headers =
        [ ("Content-Type", defaultMimeLookup path),
          ("Cache-Control", "max-age=604800, public")
        ]
  catchIf
    isDoesNotExistError
    ( do
        contents <- case List.lookup path [getBundledCSS ?ctx, getBundledJS ?ctx] of
          Just contents -> pure contents
          Nothing -> getResource $ "static/" <> fromText path
        pure (HTTP.status200, headers, contents)
    )
    (const showNotFoundPage)
