module OkVault.Router
  ( routeRequest,
  )
where

import Data.ByteString.Lazy (fromStrict)
import qualified Network.Wai as Wai
import Network.Wai.Request (appearsSecure)
import OkStd
import qualified OkVault.Actions as Actions
import OkVault.Context

routeRequest :: InContext Wai.Application
routeRequest req respond =
  let ?req = req
   in do
        (status, headers, body) <-
          if not (getAllowInsecure ?ctx || appearsSecure req)
            then Actions.showInsecureRequestPage
            else case (Wai.requestMethod req, Wai.pathInfo req) of
              ("GET", "static" : _) ->
                Actions.staticFile
              ("GET", []) ->
                Actions.showIndexPage
              ("GET", ["auth", "login"]) ->
                Actions.showLoginPage
              ("POST", ["auth", "send-code"]) ->
                Actions.sendLoginCode
              ("POST", ["auth", "validate-code"]) ->
                Actions.validateLoginCode
              ("GET", ["auth", "logout"]) ->
                Actions.showLogoutPage
              ("POST", ["auth", "logout"]) ->
                Actions.logOut
              ("GET", ["first-time"]) ->
                Actions.showFirstTimePage
              ("POST", ["first-time"]) ->
                Actions.performFirstTimeSetup
              ("GET", ["new-item"]) ->
                Actions.showNewItemPage
              ("POST", ["new-item"]) ->
                Actions.createNewItem
              ("GET", ["item", slug]) ->
                Actions.showItem slug
              ("GET", ["edit-item", slug]) ->
                Actions.showEditItemPage slug
              ("POST", ["edit-item", slug]) ->
                Actions.updateItem slug
              ("GET", ["delete-item", slug]) ->
                Actions.showDeleteItemPage slug
              ("POST", ["delete-item", slug]) ->
                Actions.deleteItem slug
              _ ->
                Actions.showNotFoundPage
        respond $ Wai.responseLBS status headers $ fromStrict body
