class AddEditItemPage {
  static exec(vault) {
    _.onShow('page', evt => {
      const item = vault.decryptItem();
      new AddEditItemPage(
        vault,
        item || Item.empty(Item.TypeLogin),
        item === null
      ).exec();
    });
    MasterPasswordPrompt.showIfNeeded(vault);
  }

  constructor(vault, item, isNew) {
    this.vault = vault;
    this.item = item;
    this.isNew = isNew;
  }

  exec() {
    if (this.isNew) {
      this.setupItemTypeSelector();
    }
    this.setupForm();
  }

  setupItemTypeSelector() {
    _.removeAllChildrenOf('item-type');
    _.elt('item-type').appendChild(
      _.cloneTemplate(`template-item-type-${this.item.type}`)
    );

    for (const itemType of Item.Types) {
      if (this.item.type === itemType) {
        continue;
      }
      _.onClick(`item-type-${itemType}`, evt =>
        new AddEditItemPage(this.vault, Item.empty(itemType), true).exec()
      );
    }
  }

  setupForm() {
    _.removeAllChildrenOf('item-form-placeholder');
    _.elt('item-form-placeholder').appendChild(
      _.cloneTemplate(`template-form-${this.item.type}`)
    );

    for (const field of Item.fields(this.item)) {
      _.setValueOf(`item-${field}`, this.item[field]);
    }

    if (this.item.type == Item.TypeLogin) {
      _.onClick('generate-password', evt =>
        _.setValueOf('item-password', Pwgen.generate()));
    }

    _.onSubmit('item-form', evt => {
      for (const field of Item.fields(this.item)) {
        this.item[field] = _.valueOf(`item-${field}`);
      }
      const {name, category, content, key} = this.vault.encryptItem(this.item);
      _.setValueOf('name', name);
      _.setValueOf('category', category);
      _.setValueOf('content', content);
      _.setValueOf('item_key', key);
    });
  }
}