class ItemsListPage {
  static exec(vault) {
    _.onShow('page', evt => {
      new ItemsListPage(
        vault,
        vault.decryptItemsList(),
      ).exec();
    });
    MasterPasswordPrompt.showIfNeeded(vault);
  }

  constructor(vault, items) {
    this.vault = vault;
    this.items = items;
  }

  exec() {
    const templateCategory = _.elt('template-category');
    const templateItem = _.elt('template-item');

    const itemsByCategory = this.items.reduce((m, item) => {
      if (!m.has(item.category)) {
        m.set(item.category, []);
      }
      m.get(item.category).push(item);
      return m;
    }, new Map());
    const categories = Array.from(itemsByCategory.keys());
    categories.sort();

    for (const category of categories) {
      _.elt('items-list').appendChild(_.cloneTemplate('template-category'));
      _.withElt('category-title', e => {
        e.innerText = category;
        e.removeAttribute('id');
      });

      const categoryItems = itemsByCategory.get(category);
      categoryItems.sort((a, b) => a.name.localeCompare(b.name));
      for (const item of categoryItems) {
        _.elt('category-items').appendChild(_.cloneTemplate('template-item'));
        _.withElt('item-link', e => {
          e.innerText = item.name;
          e.href = `/item/${item.slug}`;
          e.removeAttribute('id');
        });
      }
      _.elt('category-items').removeAttribute('id');
    }
  }
}