class ShowItemPage {
  static async exec(vault) {
    _.onShow('page', async evt =>{
      const item = vault.decryptItem();
      switch (item.type) {
        case Item.TypeLogin:
          await this.showLogin(item);
          break;
        case Item.TypeCard:
          this.showCard(item);
          break;
        case Item.TypeNote:
          this.showNote(item);
          break;
        default:
          throw new Error(`Invalid item type: ${item.type}`);
      }
    });
    MasterPasswordPrompt.showIfNeeded(vault);
  }

  static HiddenValue = '*****'
  static HiddenFields = ['password', 'securityCode', 'pinCode']

  static async showLogin(item) {
    _.elt('page').appendChild(_.cloneTemplate('template-item-login'));
    _.elt('item-link-edit').href = `/edit-item/${item.slug}`;
    _.elt('item-link-delete').href = `/delete-item/${item.slug}`;

    for (const field of Item.fields(item)) {
      if (!item[field]) {
        _.elt(`field-${field}`).hidden = true;
        continue;
      }

      if (field === 'totpSecret') {
        try {
          const updateFields = async () => {
            const passcode = await Totp.generatePasscode(item.totpSecret);
            _.withElt('totp-passcode', e => {
              e.innerText = passcode;
              e.value = passcode;
            });
            _.elt('totp-expires-in').innerText = Totp.expiresIn();
          };

          await updateFields();
          setInterval(updateFields, 1000);
          _.onClick('copy-totp-passcode', async evt =>
            navigator.clipboard.writeText(_.valueOf('totp-passcode')));
        } catch (exc) {
          console.error(exc);
          _.elt('totp-passcode').innerText = "Invalid TOTP secret";
          _.hide('totp-actions');
        }
      }
      else if (this.HiddenFields.includes(field)) {
        _.withElt(`item-${field}`, e => {
          e.value = item[field];
          e.innerText = this.HiddenValue;
          e.isValueHidden = true;
        });
        _.onClick(`toggle-${field}`, evt =>
          _.withElt(`item-${field}`, e => {
            e.isValueHidden = !e.isValueHidden;
            e.innerText = e.isValueHidden ? this.HiddenValue : e.value;
            _.setValueOf(`toggle-${field}`, e.isValueHidden ? "Show" : "Hide");
          }));
        _.onClick(`copy-${field}`, async evt =>
          navigator.clipboard.writeText(_.valueOf(`item-${field}`)));
      }
      else {
        _.elt(`item-${field}`).innerText = item[field];
        if (field === 'address') {
          _.elt(`item-${field}`).href = item[field];
        }
      }
    }
  }

  static showCard(item) {
    _.elt('page').appendChild(_.cloneTemplate('template-item-card'));
    _.elt('item-link-edit').href = `/edit-item/${item.slug}`;
    _.elt('item-link-delete').href = `/delete-item/${item.slug}`;

    for (const field of Item.fields(item)) {
      if (!item[field]) {
        _.elt(`field-${field}`).hidden = true;
        continue;
      }

      if (this.HiddenFields.includes(field)) {
        _.withElt(`item-${field}`, e => {
          e.value = item[field];
          e.innerText = this.HiddenValue;
          e.isValueHidden = true;
        });
        _.onClick(`toggle-${field}`, evt =>
          _.withElt(`item-${field}`, e => {
            e.isValueHidden = !e.isValueHidden;
            e.innerText = e.isValueHidden ? this.HiddenValue : e.value;
            _.setValueOf(`toggle-${field}`, e.isValueHidden ? "Show" : "Hide");
          }));
        _.onClick(`copy-${field}`, async evt =>
          navigator.clipboard.writeText(_.valueOf(`item-${field}`)));
      }
      else {
        _.elt(`item-${field}`).innerText = item[field];
      }
    }
  }

  static showNote(item) {
    _.elt('page').appendChild(_.cloneTemplate('template-item-note'));
    _.elt('item-link-edit').href = `/edit-item/${item.slug}`;
    _.elt('item-link-delete').href = `/delete-item/${item.slug}`;
    for (const field of Item.fields(item)) {
      if (field === 'note') {
        _.withElt('item-note', e => {
          e.value = item.note;
          e.style.height = (e.scrollHeight + 5) + 'px';
        });
      } else {
        _.elt(`item-${field}`).innerText = item[field];
      }
    }
  }
}