class Totp {
  static Period = 30;
  static Digits = 6;

  static async generatePasscode(secret) {
    const t = Math.floor(this.now() / this.Period);
    const counter = Uint8Array.of(
      0, 0, 0, 0, // FIXME figure out what to do after 2038
      (t >>> 24) & 0xFF,
      (t >>> 16) & 0xFF,
      (t >>> 8) & 0xFF,
      t & 0xFF
    );
    const hash = await this.hmacSha1(this.fromBase32(secret), counter);
    const offset = hash[hash.length - 1] & 0xF;
    const binary =
      ((hash[offset] & 0x7F) << 24) |
      ((hash[offset+1] & 0xFF) << 16) |
      ((hash[offset+2] & 0xFF) << 8) |
      (hash[offset+3] & 0xFF);
    const otp = binary % (10 ** this.Digits);

    return otp.toString().padStart(this.Digits, '0');
  }

  static expiresIn() {
    return (this.Period - (this.now() % this.Period));
  }

  static async hmacSha1(keyData, data) {
    const scrypto = window.crypto.subtle;
    const k = await scrypto.importKey(
      'raw',
      keyData,
      {name: 'HMAC', hash: 'SHA-1'},
      false,
      ['sign']);
    const s = await scrypto.sign('HMAC', k, data);
    return new Uint8Array(s);
  }

  static now() {
    return Math.floor(Date.now() / 1000);
  }

  static fromBase32(str) {
    const alphabet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ234567';
    const padding = '=';

    const padded =
      (str.length % 8 === 0) ?
      str :
      str.padEnd(8 * (Math.floor(str.length / 8) + 1), padding);

    const result = new Uint8Array((padded.length * 5) / 8);
    var cursor = 0;

    for (var i = 0; i < padded.length / 8; i++) {
      var skip = 0;

      for (var j = i * 8; j < (i + 1) * 8; j++) {
        if (padded.charAt(j) == padding) {
          break;
        }

        const val = alphabet.indexOf(padded.charAt(j)) << 3;
        if (val < 0) {
          throw new Error(`Can't decode: ${str}`);
        }

        result[cursor] |= val >>> skip;
        skip += 5;
        if (skip >= 8) {
          cursor++;
          skip -= 8;
          result[cursor] = skip > 0 ? (val << (5 - skip)) & 0xFF : 0;
        }
      }
    }

    return result.slice(0, cursor);
  }
}