class Vault {
  static async init() {
    const crypt = await Crypt.init();
    return new Vault(crypt);
  }

  constructor(crypt) {
    this.crypt = crypt;
    this.storage = window.sessionStorage;
  }

  get masterKey() {
    const key = this.storage.getItem('master-key');
    if (key === null) {
      return null;
    }
    return this.crypt.fromBase64(key);
  }

  get isLocked() {
    return this.masterKey === null;
  }

  unlock(masterPassword) {
    try {
      const salt = this.crypt.fromBase64(_.valueOf('salt'));
      const masterKey = this.crypt.deriveKey(masterPassword, salt);
      this.crypt.decrypt(_.valueOf('metadata-key'), masterKey)
      this.storage.setItem('master-key', this.crypt.toBase64(masterKey));
      return true;
    } catch (exc) {
      console.error(`Can't unlock vault: ${exc}`);
      return false;
    }
  }

  decryptMetadataKey() {
    return this.crypt.decrypt(_.valueOf('metadata-key'), this.masterKey);
  }

  decryptDataKey() {
    return this.crypt.decrypt(_.valueOf('data-key'), this.masterKey);
  }

  decryptItem() {
    const itemData = JSON.parse(_.elt('item').text);
    if (itemData === null) {
      return null;
    }
    const dataKey = this.decryptDataKey();
    const itemKey = this.crypt.decrypt(itemData.key, dataKey);
    const item = JSON.parse(this.crypt.decryptString(itemData.content, itemKey));
    item.slug = itemData.slug
    return item;
  }

  encryptItem(item) {
    const metadataKey = this.decryptMetadataKey();
    const dataKey = this.decryptDataKey();
    const itemKey = this.crypt.generateKey();
    return {
      name: this.crypt.encrypt(item.name, metadataKey),
      category: this.crypt.encrypt(item.category, metadataKey),
      content: this.crypt.encrypt(JSON.stringify(item), itemKey),
      key: this.crypt.encrypt(itemKey, dataKey),
    };
  }

  decryptItemsList() {
    const itemListData = JSON.parse(_.elt('items').text);
    if (itemListData === null) {
      return [];
    }

    const metadataKey = this.decryptMetadataKey();
    const result = [];
    for (const itemData of itemListData) {
      const item = {
        name: this.crypt.decryptString(itemData.name, metadataKey),
        category: this.crypt.decryptString(itemData.category, metadataKey),
        slug: itemData.slug,
      };
      result.push(item);
    }
    return result;
  }

  decryptItemToDelete() {
    const itemData = JSON.parse(_.elt('item').text);
    if (itemData === null) {
      return null;
    }
    const metadataKey = this.decryptMetadataKey();
    return {
      name: this.crypt.decryptString(itemData.name, metadataKey),
      slug: itemData.slug,
    };
  }
}